/*
 *
 *  Created on: Mar 1, 2021
 *      Author: astrisw
 */

#pragma once

#include <fstream>
#include <Base_Archiver.h>
#include <Base_Configurator.h>

namespace inaf::oasbo::Archivers {

class FileArchiver: public BaseArchiver {
protected:
	std::ofstream *myfile = nullptr;
	bool _open;
	std::string computeFileName(std::string fileName);
	std::string resolveEnvVar(std::string path);
	FileArchiver(std::string dest);
	FileArchiver(std::string parent, std::string dest);
	FileArchiver();
public:
	std::string parentPath;

	int write(PacketLib::BasePacket&) override;
	int write(PacketLib::BasePacket&, std::string dest) override;

	int close() override;
	int open() override;
	bool is_open() override;
	void setDest(std::string) override;
	void updateDest() override;
	std::string getDest() override;
	~FileArchiver() {
		close();
	}

	friend class FileArchiverBuilder;

};


class FileArchiverBuilder {
protected:
	FileArchiver *arch;
public:
	std::string config_target { "filearchiver" };
	std::string file_name_key { "file_name" };
	std::string archive_path_key { "archive_path" };

	FileArchiverBuilder();
	FileArchiverBuilder(std::string dest);
	FileArchiverBuilder(std::string parent, std::string dest);
	~FileArchiverBuilder();

	void reset();

	FileArchiverBuilder* configFrom(Configurators::BaseConfigurator &conf);

	FileArchiverBuilder* setParent(std::string parent);

	FileArchiverBuilder* setDest(std::string dest);

	FileArchiver* getArchiver();
};
}
